import net.sergeych.merge3.MergeResult
import net.sergeych.merge3.MergedBlock
import net.sergeych.merge3.merge3
import net.sergeych.mp_logger.Log
import net.sergeych.sprintf.sprintf
import kotlin.test.Test
import kotlin.test.assertEquals

val List<Char>.str: String get() = joinToString("")

    fun <T>MergeResult<T>.toTrace(source: List<T>): String {
        val r = StringBuilder()
        r.append(merged.indices.joinToString("") { "%3d".sprintf(it) } + "\n")
        r.append(merged.joinToString("") { "%3s".sprintf(it) } + "\n")
        r.append("---- source ----\n")
        r.append(source.joinToString("") { "%3s".sprintf(it) } + "\n")
        r.append(sourceIndices.joinToString("") { "%3s".sprintf(it) } + "\n")
        return r.toString()
    }

class BasicTest {
    @Test
    fun testMergeNoConflicts() {
        val src = "Hello world".toList()
        val a = "Hello cruel world!".toList()
        val b = "Bye world!!".toList()

        val m = merge3(src, b, a)
        println(m.merged.str)
//        println(m.conflicts)
        println(m.changedAreas)
        assertEquals("Bye cruel world!!!", m.merged.str)
//        assertTrue(m.noConflicts)
    }
    @Test
    fun testMergeNoConflicts3() {
        Log.connectConsole()
        val src = "Hello, world!".toList()
        val a = "Hello, friend!".toList()
        val b = "Bye world!".toList()

        val m = merge3(src, a, b, true)
        println(m.merged.str)
//        println(m.conflicts)
        println(m.changedAreas)
        assertEquals("Bye friend!", m.merged.str)
        println(m.toTrace(src))
//        assertTrue(m.noConflicts)
    }

    @Test
    fun testMergeNoConflicts2() {
        val src = "Hello world".toList()
        val a = "Hello friend".toList()
        val b = "Bye world".toList()

        val m = merge3(src, b, a)
        println(m.merged.str)
//        println(m.conflicts)
        println(m.changedAreas)
        assertEquals("Bye friend", m.merged.str)
//        assertTrue(m.noConflicts)
        println(m.blocks)
    }
    @Test
    fun testMergeWithConflicts() {
        Log.connectConsole()
        val src = "Hello world".toList()
        val a = "Hello 123".toList()
        val b = "Hello 456".toList()

        val m = merge3(src, b, a, false)
        println(m.merged.str)
//        println(m.conflicts)
        println(m.changedAreas)
        assertEquals("Hello 123456", m.merged.str)
    }

    @Test
    fun testMergeIndexes() {
        Log.connectConsole()
        val src = "Hello".toList()
        val a = "123 Hello".toList()
        val b = "456 Hello".toList()

        val m = merge3(src, b, a, true)
//        println(m.toTrace(src))
//        println(m.changedAreas)
//        for( b in m.blocks )
//            println(b)
        val unchanged = m.blocks.last() as MergedBlock.Unchanged
        println(unchanged)
        assertEquals(0, unchanged.referenceIndex)
    }

    @Test
    fun testMergeIndexes2() {
        Log.connectConsole()
        val src = "Hello".toList()
        val a = "1Hello".toList()
        val b = "2Hello".toList()

        val m = merge3(src, b, a, true)
        val unchanged = m.blocks.last() as MergedBlock.Unchanged
        println(unchanged)
        assertEquals(0, unchanged.referenceIndex)
    }
}